package javax.swing;

import java.beans.PropertyChangeEvent;
import java.awt.event.ComponentEvent;
import java.awt.Window;
import javax.swing.event.AncestorEvent;
import java.awt.Container;
import javax.swing.event.AncestorListener;
import javax.swing.event.EventListenerList;
import java.awt.Component;
import java.io.Serializable;
import java.beans.PropertyChangeListener;
import java.awt.event.ComponentListener;

class AncestorNotifier implements ComponentListener, PropertyChangeListener, Serializable
{
    transient Component firstInvisibleAncestor;
    EventListenerList listenerList;
    JComponent root;
    
    AncestorNotifier(JComponent root) {
        this.listenerList = new EventListenerList();
        this.addListeners(this.root = root, true);
    }
    
    void addAncestorListener(AncestorListener l) {
        this.listenerList.add(AncestorListener.class, l);
    }
    
    void removeAncestorListener(AncestorListener l) {
        this.listenerList.remove(AncestorListener.class, l);
    }
    
    AncestorListener[] getAncestorListeners() {
        return (AncestorListener[])this.listenerList.getListeners(AncestorListener.class);
    }
    
    protected void fireAncestorAdded(JComponent source, int id, Container ancestor, Container ancestorParent) {
        Object[] listenerList = this.listenerList.getListenerList();
        for (int i = listenerList.length - 2; i >= 0; i -= 2) {
            if (listenerList[i] == AncestorListener.class) {
                ((AncestorListener)listenerList[i + 1]).ancestorAdded(new AncestorEvent(source, id, ancestor, ancestorParent));
            }
        }
    }
    
    protected void fireAncestorRemoved(JComponent source, int id, Container ancestor, Container ancestorParent) {
        Object[] listenerList = this.listenerList.getListenerList();
        for (int i = listenerList.length - 2; i >= 0; i -= 2) {
            if (listenerList[i] == AncestorListener.class) {
                ((AncestorListener)listenerList[i + 1]).ancestorRemoved(new AncestorEvent(source, id, ancestor, ancestorParent));
            }
        }
    }
    
    protected void fireAncestorMoved(JComponent source, int id, Container ancestor, Container ancestorParent) {
        Object[] listenerList = this.listenerList.getListenerList();
        for (int i = listenerList.length - 2; i >= 0; i -= 2) {
            if (listenerList[i] == AncestorListener.class) {
                ((AncestorListener)listenerList[i + 1]).ancestorMoved(new AncestorEvent(source, id, ancestor, ancestorParent));
            }
        }
    }
    
    void removeAllListeners() {
        this.removeListeners(this.root);
    }
    
    void addListeners(Component parent, boolean b) {
        this.firstInvisibleAncestor = null;
        for (; this.firstInvisibleAncestor == null; parent = parent.getParent()) {
            if (b || parent != parent) {
                parent.addComponentListener(this);
                if ((parent instanceof JComponent)) {
                    ((JComponent)parent).addPropertyChangeListener(this);
                }
            }
            if (!parent.isVisible() || parent.getParent() == null || parent instanceof Window) {
                this.firstInvisibleAncestor = parent;
            }
        }
        if (this.firstInvisibleAncestor instanceof Window && this.firstInvisibleAncestor.isVisible()) {
            this.firstInvisibleAncestor = null;
        }
    }
    
    void removeListeners(Component parent) {
        while (parent != null) {
            parent.removeComponentListener(this);
            if ((parent instanceof JComponent)) {
                ((Component)parent).removePropertyChangeListener(this);
            }
            if (parent != this.firstInvisibleAncestor) {
                if ((parent instanceof Window)) {
                    break;
                }
                parent = parent.getParent();
            }
            else {
                break;
            }
        }
    }
    
    @Override
    public void componentResized(ComponentEvent componentEvent) {
    }
    
    @Override
    public void componentMoved(ComponentEvent componentEvent) {
        Component component = componentEvent.getComponent();
        this.fireAncestorMoved(this.root, 3, (Container)component, component.getParent());
    }
    
    @Override
    public void componentShown(ComponentEvent componentEvent) {
        Component component = componentEvent.getComponent();
        if (component == this.firstInvisibleAncestor) {
            this.addListeners(component, false);
            if (this.firstInvisibleAncestor == null) {
                this.fireAncestorAdded(this.root, 1, (Container)component, component.getParent());
            }
        }
    }
    
    @Override
    public void componentHidden(ComponentEvent componentEvent) {
        Component component = componentEvent.getComponent();
        int n = this.firstInvisibleAncestor == null;
        if (!(component instanceof Window)) {
            this.removeListeners(component.getParent());
        }
        this.firstInvisibleAncestor = component;
        if (!(n == 0)) {
            this.fireAncestorRemoved(this.root, 2, (Container)component, component.getParent());
        }
    }
    
    @Override
    public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
        String propertyName = propertyChangeEvent.getPropertyName();
        if (propertyName != null && (propertyName.equals("parent") || propertyName.equals("ancestor"))) {
            JComponent firstInvisibleAncestor = (JComponent)propertyChangeEvent.getSource();
            if (propertyChangeEvent.getNewValue() != null) {
                if (firstInvisibleAncestor == this.firstInvisibleAncestor) {
                    this.addListeners(firstInvisibleAncestor, false);
                    if (this.firstInvisibleAncestor == null) {
                        this.fireAncestorAdded(this.root, 1, firstInvisibleAncestor, firstInvisibleAncestor.getParent());
                    }
                }
            }
            else {
                int n = this.firstInvisibleAncestor == null;
                Container container = (Container)propertyChangeEvent.getOldValue();
                this.removeListeners(container);
                this.firstInvisibleAncestor = firstInvisibleAncestor;
                if (!(n == 0)) {
                    this.fireAncestorRemoved(this.root, 2, firstInvisibleAncestor, container);
                }
            }
        }
    }
}
